@@ -1,7 +1,8 @@
 import os
 import sys
-import monitor
+import qtask.monitor as monitor
 import re
+import subprocess
 
 QTASK_MON = os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "bin", "qtask-mon"))
 
@@ -23,25 +24,34 @@ Valid job resource/arguments:
 Note: These values are all job-scheduler dependent
 '''
 
-    def __init__(self, cmd, name=None, resources=None):
+    def __init__(self, cmd, name=None, resources=None, skip=False):
         self.name = name
         self.cmd = cmd
-        self.resources = {'env': True, 'wd': os.path.abspath(os.curdir)}
-        for k in resources:
-            self.resources[k] = resources[k]
+        self.skip = skip
+        self.resources = {'env': True, 'wd': os.path.abspath(os.curdir), 'force_first': False}
+        if resources:
+            for k in resources:
+                self.resources[k] = resources[k]
 
         self.jobid = None
         self.runner = None
         self.basename = None
         self.depends = []
 
+    def __nonzero__(self):
+        return not self.skip
+
     def deps(self, *deps):
         for d in deps:
-            if d:
+            if d and not d.skip:
                 self.depends.append(d)
 
         return self
 
+    def set_name(self, name):
+        self.name = name
+        return self
+
     @property
     def fullname(self):
         if self.basename:
@@ -65,8 +75,15 @@ def task(**task_args):
             else:
                 cmd = ret
 
+            if 'progs' in context:
+                if type(context['progs']) == type(''):
+                    check_path(context['progs'])
+                else:
+                    for prog in context['progs']:
+                        check_path(prog)
+
             if not cmd and not 'holding' in context:
-                return None
+                return QTask('', skip=True)
 
             if 'name' in kwargs:
                 name = kwargs['name']
@@ -75,12 +92,32 @@ def task(**task_args):
                 name = func.__name__
 
             task = QTask(cmd, name, context)
+
+            if not cmd and not 'holding' in context:
+                task = QTask('', name, skip=True)
+            else:
+                task = QTask(cmd, name, context)
             pipeline.add_task(task)
             return task
         return wrapped_func
     return wrap
 
 
+__path_cache = set()
+def check_path(prog):
+    if prog in __path_cache:
+        return True
+
+    with open('/dev/null', 'w') as devnull:
+        if subprocess.call("which %s" % prog, stderr=devnull, stdout=devnull, shell=True) != 0:
+            raise RuntimeError("Missing required program from $PATH: %s\n\n" % prog)
+
+    __path_cache.add(prog)
+    return True
+
+
+
+
 class JobRunner(object):
     def done(self):
         pass
@@ -92,6 +129,7 @@ class JobRunner(object):
         raise NotImplementedError
 
     def qsub(self, task, monitor, verbose=False, dryrun=False):
+        'return a tuple: (jobid, script_src)'
         raise NotImplementedError
 
 
@@ -125,7 +163,7 @@ class BashRunner(JobRunner):
             self.script += '%s\n' % task.cmd
 
         self._jobid += 1
-        return jobid
+        return jobid, ''
 
     def qdel(self, jobid):
         pass
@@ -137,6 +175,11 @@ class BashRunner(JobRunner):
         print self.script
 
 
+@task(holding=True, force_first=True)
+def holding():
+    return ''
+
+
 class __Pipeline(object):
     def __init__(self):
         self._reset()
@@ -169,6 +212,7 @@ class __Pipeline(object):
         self.project = ''
         self.sample = ''
         self._submitted_tasks = set()
+        self.global_depends = []
 
 
     @property
@@ -209,18 +253,25 @@ class __Pipeline(object):
                 basename = re.sub(r'\s', '_', self.project)
         
         task.basename = basename
+        task.deps(*self.global_depends)
+
+        if task.resources['force_first']:
+            self.tasks.insert(0, task)
+        else:
+            self.tasks.append(task)
 
-        self.tasks.append(task)
 
     def submit(self, verbose=False, dryrun=False):
         mon = None
         if not dryrun and self.config['monitor']:
             mon = monitor.load_monitor(self.config['monitor'])
 
-        while len(self._submitted_tasks) != len(self.tasks):
+        while self.tasks:
             for t in self.tasks:
-                # skip if we've already submitted it
-                if t.jobid:
+                # is this an already processed job (flagged to skip)
+                if t.skip:
+                    sys.stderr.write(' - skipped %s\n' % t.name)
+                    self._submitted_tasks.add(t)
                     continue
 
                 # check to see if dependencies have been submitted
@@ -234,14 +285,14 @@ class __Pipeline(object):
 
                 # submit
                 try:
-                    jobid = self.runner.qsub(t, monitor=self.config['monitor'], verbose=verbose, dryrun=dryrun)
+                    jobid, src = self.runner.qsub(t, monitor=self.config['monitor'], verbose=verbose, dryrun=dryrun)
                     t.jobid = jobid
                     t.runner = self.runner
                     self._submitted_tasks.add(t)
-                    sys.stderr.write('%s %s\n' % (jobid, t.name))
+                    sys.stderr.write('%s %s (%s)\n' % (jobid, t.name, ','.join([d.jobid for d in t.depends])))
 
                     if mon and not dryrun:
-                        mon.submit(jobid, t.name, t.resources['ppn'] if 'ppn' in t.resources else '', t.resources['mem'] if 'mem' in t.resources else '', src=t.cmd, project=self.project, sample=self.sample)
+                        mon.submit(jobid, t.name, procs=t.resources['ppn'] if 'ppn' in t.resources else 1, deps=[x.jobid for x in t.depends], src=src, project=self.project, sample=self.sample)
                         # subprocess.call([os.path.join(os.path.dirname(__file__), "..", "bin", "qtask-mon"), self.config['monitor'], "submit", str(jobid), t.name], shell=True)
 
                 except RuntimeError, e:
@@ -249,16 +300,23 @@ class __Pipeline(object):
                     # there was a problem...
                     self.abort()
                     raise e
+            
+            remaining = []
+            for t in self.tasks:
+                if t not in self._submitted_tasks:
+                    remaining.append(t)
+            self.tasks = remaining
         self.runner.done()
         if mon:
             mon.close()
 
-        # victoire! reset pipeline
+        # victoire! reset pipeline and release the hounds!
         self._reset()
 
     def abort(self):
         for t in self._submitted_tasks:
-            self.runner.qdel(t.jobid)
+            if not t.skip:
+                self.runner.qdel(t.jobid)
 
 
 pipeline = __Pipeline()
